"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var path_1 = __importDefault(require("path"));
var createControllersText_1 = __importDefault(require("./createControllersText"));
var checkRequisites_1 = __importDefault(require("./checkRequisites"));
var genHandlerText = function (isAsync) { return "\nconst ".concat(isAsync ? 'asyncM' : 'm', "ethodToHandler = (\n  methodCallback: ServerHandler").concat(isAsync ? 'Promise' : '', "<any, any>\n): RequestHandler => ").concat(isAsync ? 'async ' : '', "(req, res, next) => {\n  try {\n    const data = ").concat(isAsync ? 'await ' : '', "methodCallback(req as any) as any\n\n    if (data.headers) {\n      for (const key in data.headers) {\n        res.setHeader(key, data.headers[key])\n      }\n    }\n\n    res.status(data.status).send(data.body)\n  } catch (e) {\n    next(e)\n  }\n}\n"); };
var genHandlerWithSchemaText = function (isAsync) { return "\nconst ".concat(isAsync ? 'asyncM' : 'm', "ethodToHandlerWithSchema = (\n  methodCallback: ServerHandler").concat(isAsync ? 'Promise' : '', "<any, any>,\n  schema: { [K in HttpStatusOk]?: Schema }\n): RequestHandler => {\n  const stringifySet = Object.entries(schema).reduce(\n    (prev, [key, val]) => ({ ...prev, [key]: fastJson(val!) }),\n    {} as Record<HttpStatusOk, ReturnType<typeof fastJson> | undefined>\n  )\n\n  return ").concat(isAsync ? 'async ' : '', "(req, res, next) => {\n    try {\n      const data = ").concat(isAsync ? 'await ' : '', "methodCallback(req as any) as any\n      const stringify = stringifySet[data.status as HttpStatusOk]\n\n      if (stringify) {\n        res.set('content-type', 'application/json; charset=utf-8')\n\n        if (data.headers) {\n          for (const key in data.headers) {\n            res.setHeader(key, data.headers[key])\n          }\n        }\n\n        res.status(data.status).send(stringify(data.body))\n      } else {\n        if (data.headers) {\n          for (const key in data.headers) {\n            res.setHeader(key, data.headers[key])\n          }\n        }\n\n        res.status(data.status).send(data.body)\n      }\n    } catch (e) {\n      next(e)\n    }\n  }\n}\n"); };
exports.default = (function (input, project) {
    var _a = (0, createControllersText_1.default)("".concat(input, "/api"), project !== null && project !== void 0 ? project : input), imports = _a.imports, consts = _a.consts, controllers = _a.controllers;
    var hasNumberTypeQuery = controllers.includes('parseNumberTypeQueryParams(');
    var hasBooleanTypeQuery = controllers.includes('parseBooleanTypeQueryParams(');
    var hasOptionalQuery = controllers.includes('  callParserIfExistsQuery(');
    var hasJSONBody = controllers.includes('  parseJSONBoby,');
    var hasTypedParams = controllers.includes('  createTypedParamsHandler(');
    var hasValidator = controllers.includes('  validateOrReject(');
    var hasMulter = controllers.includes('  uploader,');
    var hasMethodToHandler = controllers.includes(' methodToHandler(');
    var hasAsyncMethodToHandler = controllers.includes(' asyncMethodToHandler(');
    var hasMethodToHandlerWithSchema = controllers.includes(' methodToHandlerWithSchema(');
    var hasAsyncMethodToHandlerWithSchema = controllers.includes(' asyncMethodToHandlerWithSchema(');
    var hasValidatorCompiler = controllers.includes(' validatorCompiler');
    var headImports = [];
    (0, checkRequisites_1.default)({ hasValidator: hasValidator });
    if (controllers.includes('response: responseSchema')) {
        console.warn("frourio-express: 'responseSchema' is deprecated. Specify schemas.response in controller instead.");
    }
    if (controllers.includes('ctrlHooks0.')) {
        console.warn("frourio-express: 'defineHooks in controller.ts' is deprecated. Specify hooks in controller instead.");
    }
    if (hasValidator) {
        console.warn("frourio-express: 'class-validator' is deprecated. Specify validators in controller instead. ref: https://frourio.com/docs/reference/validation/zod");
        headImports.push("import 'reflect-metadata'", "import type { ClassTransformOptions } from 'class-transformer'", "import { plainToInstance as defaultPlainToInstance  } from 'class-transformer'", "import type { ValidatorOptions } from 'class-validator'", "import { validateOrReject as defaultValidateOrReject } from 'class-validator'");
    }
    if (hasMulter) {
        headImports.push("import path from 'path'");
    }
    headImports.push("import type { Express, RequestHandler".concat(hasValidator ? ', Request' : '', " } from 'express'"));
    if (hasJSONBody) {
        headImports.push("import express from 'express'");
    }
    if (hasMulter) {
        headImports.push("import type { Options } from 'multer'");
        headImports.push("import multer from 'multer'");
    }
    if (hasMethodToHandlerWithSchema || hasAsyncMethodToHandlerWithSchema) {
        headImports.push("import fastJson from 'fast-json-stringify'");
    }
    if (hasValidator) {
        headImports.push("import * as Validators from './validators'");
    }
    if (hasMulter) {
        headImports.push("import type { ReadStream } from 'fs'");
    }
    headImports.push("import type { HttpStatusOk, AspidaMethodParams } from 'aspida'");
    return {
        text: "".concat(headImports.join('\n'), "\nimport type { Schema } from 'fast-json-stringify'\nimport type { z } from 'zod'\n").concat(imports, "\n\nexport type FrourioOptions = {\n  basePath?: string\n").concat(hasValidator
            ? '  transformer?: ClassTransformOptions\n' +
                '  validator?: ValidatorOptions\n' +
                '  plainToInstance?: ((cls: new (...args: any[]) => object, object: unknown, options: ClassTransformOptions) => object)\n' +
                '  validateOrReject?: ((instance: object, options: ValidatorOptions) => Promise<void>)\n'
            : '').concat(hasMulter
            ? "  multer?: Options\n}\n\nexport type MulterFile = Express.Multer.File"
            : '}', "\n\ntype HttpStatusNoOk = 301 | 302 | 400 | 401 | 402 | 403 | 404 | 405 | 406 | 409 | 500 | 501 | 502 | 503 | 504 | 505\n\ntype PartiallyPartial<T, K extends keyof T> = Omit<T, K> & Partial<Pick<T, K>>\n\ntype BaseResponse<T, U, V> = {\n  status: V extends number ? V : HttpStatusOk\n  body: T\n  headers: U\n}\n\ntype ServerResponse<K extends AspidaMethodParams> =\n  | (K extends { resBody: K['resBody']; resHeaders: K['resHeaders'] }\n  ? BaseResponse<K['resBody'], K['resHeaders'], K['status']>\n  : K extends { resBody: K['resBody'] }\n  ? PartiallyPartial<BaseResponse<K['resBody'], K['resHeaders'], K['status']>, 'headers'>\n  : K extends { resHeaders: K['resHeaders'] }\n  ? PartiallyPartial<BaseResponse<K['resBody'], K['resHeaders'], K['status']>, 'body'>\n  : PartiallyPartial<\n      BaseResponse<K['resBody'], K['resHeaders'], K['status']>,\n      'body' | 'headers'\n    >)\n  | PartiallyPartial<BaseResponse<any, any, HttpStatusNoOk>, 'body' | 'headers'>\n").concat(hasMulter
            ? "\ntype BlobToFile<T extends AspidaMethodParams> = T['reqFormat'] extends FormData\n  ? {\n      [P in keyof T['reqBody']]: Required<T['reqBody']>[P] extends Blob | ReadStream\n        ? MulterFile\n        : Required<T['reqBody']>[P] extends (Blob | ReadStream)[]\n        ? MulterFile[]\n        : T['reqBody'][P]\n    }\n  : T['reqBody']\n"
            : '', "\ntype RequestParams<T extends AspidaMethodParams> = Pick<{\n  query: T['query']\n  body: ").concat(hasMulter ? 'BlobToFile<T>' : "T['reqBody']", "\n  headers: T['reqHeaders']\n}, {\n  query: Required<T>['query'] extends {} | null ? 'query' : never\n  body: Required<T>['reqBody'] extends {} | null ? 'body' : never\n  headers: Required<T>['reqHeaders'] extends {} | null ? 'headers' : never\n}['query' | 'body' | 'headers']>\n\ntype ServerHandler<T extends AspidaMethodParams, U extends Record<string, any> = {}> = (\n  req: RequestParams<T> & U\n) => ServerResponse<T>\n\ntype ServerHandlerPromise<T extends AspidaMethodParams, U extends Record<string, any> = {}> = (\n  req: RequestParams<T> & U\n) => Promise<ServerResponse<T>>\n\nexport type ServerMethodHandler<T extends AspidaMethodParams,  U extends Record<string, any> = {}> = ServerHandler<T, U> | ServerHandlerPromise<T, U> | {\n  validators?: Partial<{ [Key in keyof RequestParams<T>]?: z.ZodType<RequestParams<T>[Key]>}>\n  schemas?: { response?: { [V in HttpStatusOk]?: Schema }}\n  hooks?: {\n    onRequest?: RequestHandler | RequestHandler[]\n    preParsing?: RequestHandler | RequestHandler[]\n    preValidation?: RequestHandler | RequestHandler[]\n    preHandler?: RequestHandler | RequestHandler[]\n  }\n  handler: ServerHandler<T, U> | ServerHandlerPromise<T, U>\n}\n").concat(hasNumberTypeQuery
            ? "\nconst parseNumberTypeQueryParams = (numberTypeParams: [string, boolean, boolean][]): RequestHandler => ({ query }, res, next) => {\n  for (const [key, isOptional, isArray] of numberTypeParams) {\n    const param = query[key]\n\n    if (isArray) {\n      if (!isOptional && param === undefined) {\n        query[key] = []\n      } else if (!isOptional || param !== undefined) {\n        if (!Array.isArray(param)) return res.sendStatus(400)\n\n        const vals = (param as string[]).map(Number)\n\n        if (vals.some(isNaN)) return res.sendStatus(400)\n\n        query[key] = vals as any\n      }\n    } else if (!isOptional || param !== undefined) {\n      const val = Number(param)\n\n      if (isNaN(val)) return res.sendStatus(400)\n\n      query[key] = val as any\n    }\n  }\n\n  next()\n}\n"
            : '').concat(hasBooleanTypeQuery
            ? "\nconst parseBooleanTypeQueryParams = (booleanTypeParams: [string, boolean, boolean][]): RequestHandler => ({ query }, res, next) => {\n  for (const [key, isOptional, isArray] of booleanTypeParams) {\n    const param = query[key]\n\n    if (isArray) {\n      if (!isOptional && param === undefined) {\n        query[key] = []\n      } else if (!isOptional || param !== undefined) {\n        if (!Array.isArray(param)) return res.sendStatus(400)\n\n        const vals = (param as string[]).map(p => p === 'true' ? true : p === 'false' ? false : null)\n\n        if (vals.some(v => v === null)) return res.sendStatus(400)\n\n        query[key] = vals as any\n      }\n    } else if (!isOptional || param !== undefined) {\n      const val = param === 'true' ? true : param === 'false' ? false : null\n\n      if (val === null) return res.sendStatus(400)\n\n      query[key] = val as any\n    }\n  }\n\n  next()\n}\n"
            : '').concat(hasOptionalQuery
            ? "\nconst callParserIfExistsQuery = (parser: RequestHandler): RequestHandler => (req, res, next) =>\n  Object.keys(req.query).length ? parser(req, res, next) : next()\n"
            : '').concat(hasJSONBody
            ? "\nconst parseJSONBoby: RequestHandler = (req, res, next) => {\n  express.json()(req, res, err => {\n    if (err) return res.sendStatus(400)\n\n    next()\n  })\n}\n"
            : '').concat(hasTypedParams
            ? "\nconst createTypedParamsHandler = (numberTypeParams: string[]): RequestHandler => (req, res, next) => {\n  const params: Record<string, string | number> = req.params\n\n  for (const key of numberTypeParams) {\n    const val = Number(params[key])\n\n    if (isNaN(val)) return res.sendStatus(400)\n\n    params[key] = val\n  }\n\n  next()\n}\n"
            : '').concat(hasValidator
            ? "\nconst createValidateHandler = (validators: (req: Request) => (Promise<void> | null)[]): RequestHandler =>\n  (req, res, next) => Promise.all(validators(req)).then(() => next()).catch(err => res.status(400).send(err))\n"
            : '').concat(hasMulter
            ? "\nconst formatMulterData = (arrayTypeKeys: [string, boolean][]): RequestHandler => ({ body, files }, _res, next) => {\n  for (const [key] of arrayTypeKeys) {\n    if (body[key] === undefined) body[key] = []\n    else if (!Array.isArray(body[key])) {\n      body[key] = [body[key]]\n    }\n  }\n\n  for (const file of files as MulterFile[]) {\n    if (Array.isArray(body[file.fieldname])) {\n      body[file.fieldname].push(file)\n    } else {\n      body[file.fieldname] = file\n    }\n  }\n\n  for (const [key, isOptional] of arrayTypeKeys) {\n    if (!body[key].length && isOptional) delete body[key]\n  }\n\n  next()\n}\n"
            : '').concat(hasValidatorCompiler
            ? "\nconst validatorCompiler = (key: 'params' | 'query' | 'headers' | 'body', validator: z.ZodType<any>): RequestHandler =>\n  (req, res, next) => {\n    const result = validator.safeParse(req[key])\n\n    if (result.success) {\n      req[key] = result.data\n      next()\n    } else {\n      res.status(400).send(result.error)\n    }\n  }\n"
            : '').concat(hasMethodToHandler ? genHandlerText(false) : '').concat(hasAsyncMethodToHandler ? genHandlerText(true) : '').concat(hasMethodToHandlerWithSchema ? genHandlerWithSchemaText(false) : '').concat(hasAsyncMethodToHandlerWithSchema ? genHandlerWithSchemaText(true) : '', "\nexport default (app: Express, options: FrourioOptions = {}) => {\n  const basePath = options.basePath ?? ''\n").concat(hasValidator
            ? '  const transformerOptions: ClassTransformOptions = { enableCircularCheck: true, ...options.transformer }\n' +
                '  const validatorOptions: ValidatorOptions = { validationError: { target: false }, ...options.validator }\n' +
                '  const { plainToInstance = defaultPlainToInstance as NonNullable<FrourioOptions["plainToInstance"]>, validateOrReject = defaultValidateOrReject as NonNullable<FrourioOptions["validateOrReject"]> } = options\n'
            : '').concat(consts).concat(hasMulter
            ? "  const uploader = multer({ dest: path.join(__dirname, '.upload'), limits: { fileSize: 1024 ** 3 }, ...options.multer }).any()\n"
            : '', "\n").concat(controllers, "\n  return app\n}\n"),
        filePath: path_1.default.posix.join(input, '$server.ts')
    };
});
//# sourceMappingURL=buildServerFile.js.map